#version 330 core

struct Light {
    vec3 direction;

	float shininess;

    vec3 ambient;
    vec3 diffuse;
    vec3 specular;       
};

in VS_OUT {
	vec3 FragPos;
	vec3 Normal;
	vec2 TexCoord;
	vec4 FragPosLightSpace;
} fs_in;

uniform sampler2D diffuseTexture;
uniform sampler2DShadow shadowMap;
uniform vec3 viewPos;
uniform Light light;

layout(location = 0) out vec4 FragColor;
layout(location = 1) out vec4 BrightColor;

void ColorOutputMRT(vec4 color)
{
	FragColor = color;
	if(dot(color.rgb,vec3(0.2126,0.7152,0.0722)) > 1.0f)
	{
		BrightColor = vec4(color.rgb, 1.0f);
	}
}

float ShadowCalculation(vec4 fragPosLightSpace)
{
	return textureProj(shadowMap, fragPosLightSpace);
}

void main()
{
	vec3 diffTexColor = vec3(texture(diffuseTexture, fs_in.TexCoord));
	vec3 normal = normalize(fs_in.Normal);
	vec3 viewDir = normalize(viewPos - fs_in.FragPos);
	////ambient
	vec3 ambient = light.ambient * diffTexColor;
	////diffuse
	float diff = max(dot(normal, light.direction), 0.0f);
	vec3 diffuse = light.diffuse * diff * diffTexColor ;
	////specular
	vec3 halfwayDir = normalize(light.direction + viewDir);
	float spec = pow(max(dot(normal, halfwayDir), 0.0f), light.shininess);
	vec3 specular = light.specular * spec * diffTexColor; // change to specTexColor
	// Calculate shadow
    float shadow = ShadowCalculation(fs_in.FragPosLightSpace);       

	ColorOutputMRT(vec4(ambient + shadow * (diffuse + specular),1.0f));
}
