#version 330 core

in vec2 TexCoords;

out vec4 outColor;

uniform sampler2D screenTexture;
uniform sampler2D bloomTexture;

const float A = 0.15f; // shoulder strength
const float B = 0.45f; // linear strength
const float C = 0.15f; // linear angle
const float D = 0.16f; // toe strenght
const float E = 0.026f; // toe numerator
const float F = 0.27f; // toe denominator (E/F toe angle)
const float W = 10.f; // linear white point value
const float exposure = 14.f;
const float verticalFringeR = 0.05f;
const float verticalFringeG = 0.03f;
const float horizontalFringeR = 0.05f;
const float horizontalFringeG = 0.03f;

const float whiteScale = ((W*(A*W+C*B)+D*E)/(W*(A*W+B)+D*F))-E/F;


vec3 COR3Tonemap(in vec3 color)
{
	vec3 x = color*exposure;
	vec3 colorMapped = ((x*(A*x+C*B)+D*E)/(x*(A*x+B)+D*F))-E/F;
	return colorMapped*whiteScale;
}

void main()
{
	const float gamma = 2.2f;
	float fringeFactorX = TexCoords.x - 0.5f;
	float fringeFunctionX = fringeFactorX*fringeFactorX*fringeFactorX;
	float fringeFactorY = TexCoords.y - 0.5f;
	float fringeFunctionY = fringeFactorY*fringeFactorY*fringeFactorY;

	vec3 colorHDR;
	colorHDR.r = texture(screenTexture, vec2(TexCoords.x-(fringeFunctionX*horizontalFringeR) ,TexCoords.y-(fringeFunctionY*verticalFringeR))).r;

	colorHDR.g = texture(screenTexture, vec2(TexCoords.x-(fringeFunctionX*horizontalFringeG) ,TexCoords.y-(fringeFunctionY*verticalFringeG))).g;

	colorHDR.b = texture(screenTexture, TexCoords).b;

	vec3 colorBloom = texture(bloomTexture, TexCoords).rgb;
	colorHDR += colorBloom;

	vec3 color = COR3Tonemap(colorHDR);

	color = pow(color, vec3(1.0f / gamma));
	outColor = vec4(color, 1.0f);
}
