#version 330 core

in vec2 TexCoords;

out vec4 outColor;

uniform sampler2D screenTexture;

const float A = 0.15f; // shoulder strength
const float B = 0.45f; // linear strength
const float C = 0.15f; // linear angle
const float D = 0.16f; // toe strenght
const float E = 0.026f; // toe numerator
const float F = 0.27f; // toe denominator (E/F toe angle)
const float W = 10.f; // linear white point value
const float exposure = 14.f;

const float whiteScale = ((W*(A*W+C*B)+D*E)/(W*(A*W+B)+D*F))-E/F;

vec3 COR3Tonemap(in vec3 color)
{
	vec3 x = color*exposure;
	vec3 colorMapped = ((x*(A*x+C*B)+D*E)/(x*(A*x+B)+D*F))-E/F;
	return colorMapped*whiteScale;
}

void main()
{

	const float gamma = 2.2f;
	vec3 colorHDR = texture(screenTexture, TexCoords).rgb;
	// reinhard
	//vec3 color = colorHDR / (colorHDR + vec3(1.0)); 

	vec3 color = COR3Tonemap(colorHDR);
	color = pow(color, vec3(1.0f / gamma));
	outColor = vec4(color, 1.0f);
	
	/*
	//this code is for rendering the depth buffer of the shadow map(debugging)...do not delete!
	float depthValue = texture(screenTexture, TexCoords).r;
	outColor = vec4(vec3(depthValue),1.0);
	*/
}
